#ifndef MINIMUMMAPPING_HH_
#define MINIMUMMAPPING_HH_

#include <ell/State.hh>

namespace ell {

	/**
	 * Defines a mapping function of states to local minima of the landscape.
	 * 
	 * @author Martin Mann
	 */
	class MinimumMapping {
		
	public:
		
		virtual ~MinimumMapping() {}
		
		//! Gives the mapping of a State to a local minimum.
		//! @param toMap the State to map
		//! @return the local minimum toMap was projected to
		virtual
		State*
		getLocalMin( const State * const toMap ) const = 0;
		
	};
	
} // namespace ell


namespace ell {
	
	  /*!
	   * Maps a given ell::State via a gradient walk to a local minimum of the
	   * energy landscape.
	   *  
	   * @author Martin Mann
	   */
	class MM_GradientWalk : public MinimumMapping {
		
	public:
		//! Creates a gradient walk minimum mapping.
		MM_GradientWalk();
		//! destruction
		virtual ~MM_GradientWalk();
		
		//! Gives the local minimum that is reached via a gradient walk.
		//! @param toMap the State to map
		//! @return the local minimum toMap was projected to
		virtual
		State*
		getLocalMin( const State * const toMap ) const;
		
	};

} // namespace ell

#endif /*MINIMUMMAPPING_HH_*/
